@@ -1,20 +1,20 @@
 /*
- * Permission is hereby granted, free of charge, to any person obtaining a copy of 
- * this software and associated documentation files (the "Software"), to deal in 
- * the Software without restriction, including without limitation the rights to 
- * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy of
+ * this software and associated documentation files (the "Software"), to deal in
+ * the Software without restriction, including without limitation the rights to
+ * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
  * of the Software, and to permit persons to whom the Software is furnished to do
  * so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included in all
  * copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
 package org.joni;
@@ -44,6 +44,11 @@ import org.joni.constants.OPSize;
 import org.joni.constants.TargetInfo;
 
 final class ArrayCompiler extends Compiler {
+    private int[]code;
+    private int codeLength;
+
+    private byte[][]templates;
+    private int templateNum;
 
     ArrayCompiler(Analyser analyser) {
         super(analyser);
@@ -51,21 +56,26 @@ final class ArrayCompiler extends Compiler {
 
     @Override
     protected final void prepare() {
-        regex.code = new int[(analyser.stop - analyser.p) * 2 + 1]; // (+1: empty regex)        
-        regex.codeLength = 0;
+        int codeSize = Config.USE_STRING_TEMPLATES ? 8 : ((analyser.getEnd() - analyser.getBegin()) * 2 + 2);
+        code = new int[codeSize];
+        codeLength = 0;
     }
 
     @Override
     protected final void finish() {
         addOpcode(OPCode.END);
         addOpcode(OPCode.FINISH); // for stack bottom
-        
+
+        regex.code = code;
+        regex.codeLength = codeLength;
+        regex.templates = templates;
+        regex.templateNum = templateNum;
+        regex.factory = MatcherFactory.DEFAULT;
+
         if (Config.USE_SUBEXP_CALL && analyser.env.unsetAddrList != null) {
             analyser.env.unsetAddrList.fix(regex);
             analyser.env.unsetAddrList = null;
         }
-
-        regex.factory = MatcherFactory.DEFAULT;
     }
 
     @Override
@@ -80,17 +90,18 @@ final class ArrayCompiler extends Compiler {
             }
         } while ((aln = aln.cdr) != null);
 
-        int pos = regex.codeLength + len;  /* goal position */
+        int pos = codeLength + len;  /* goal position */
 
         aln = node;
         do {
             len = compileLengthTree(aln.car);
             if (aln.cdr != null) {
+                regex.requireStack = true;
                 addOpcodeRelAddr(OPCode.PUSH, len + OPSize.JUMP);
             }
             compileTree(aln.car);
             if (aln.cdr != null) {
-                len = pos - (regex.codeLength + OPSize.JUMP);
+                len = pos - (codeLength + OPSize.JUMP);
                 addOpcodeRelAddr(OPCode.JUMP, len);
             }
         } while ((aln = aln.cdr) != null);
@@ -105,9 +116,13 @@ final class ArrayCompiler extends Compiler {
                 op == OPCode.EXACTN_IC_SB;
     }
 
+    private boolean opTemplated(int op) {
+        return isNeedStrLenOpExact(op);
+    }
+
     private int selectStrOpcode(int mbLength, int strLength, boolean ignoreCase) {
         int op;
-        
+
         if (ignoreCase) {
             switch(strLength) {
             case 1: op = enc.toLowerCaseTable() != null ? OPCode.EXACT1_IC_SB : OPCode.EXACT1_IC; break;
@@ -135,6 +150,7 @@ final class ArrayCompiler extends Compiler {
                 break;
             case 3:
                 op = OPCode.EXACTMB3N;
+                break;
             default:
                 op = OPCode.EXACTMBN;
             } // switch
@@ -144,15 +160,16 @@ final class ArrayCompiler extends Compiler {
 
     private void compileTreeEmptyCheck(Node node, int emptyInfo) {
         int savedNumNullCheck = regex.numNullCheck;
-        
+
         if (emptyInfo != 0) {
+            regex.requireStack = true;
             addOpcode(OPCode.NULL_CHECK_START);
             addMemNum(regex.numNullCheck); /* NULL CHECK ID */
             regex.numNullCheck++;
         }
-        
+
         compileTree(node);
-        
+
         if (emptyInfo != 0) {
             switch(emptyInfo) {
             case TargetInfo.IS_EMPTY:
@@ -165,20 +182,23 @@ final class ArrayCompiler extends Compiler {
                 addOpcode(OPCode.NULL_CHECK_END_MEMST_PUSH);
                 break;
             } // switch
-            
+
             addMemNum(savedNumNullCheck); /* NULL CHECK ID */
         }
     }
 
     private int addCompileStringlength(byte[]bytes, int p, int mbLength, int strLength, boolean ignoreCase) {
         int op = selectStrOpcode(mbLength, strLength, ignoreCase);
-        
         int len = OPSize.OPCODE;
 
+        if (Config.USE_STRING_TEMPLATES && opTemplated(op)) {
+            // string length, template index, template string pointer
+            len += OPSize.LENGTH + OPSize.INDEX + OPSize.INDEX;
+        } else {
+            if (isNeedStrLenOpExact(op)) len += OPSize.LENGTH;
+            len += mbLength * strLength;
+        }
         if (op == OPCode.EXACTMBN) len += OPSize.LENGTH;
-        if (isNeedStrLenOpExact(op)) len += OPSize.LENGTH;
-        
-        len += mbLength * strLength;
         return len;
     }
 
@@ -186,34 +206,41 @@ final class ArrayCompiler extends Compiler {
     protected final void addCompileString(byte[]bytes, int p, int mbLength, int strLength, boolean ignoreCase) {
         int op = selectStrOpcode(mbLength, strLength, ignoreCase);
         addOpcode(op);
-        
+
         if (op == OPCode.EXACTMBN) addLength(mbLength);
-        
+
         if (isNeedStrLenOpExact(op)) {
             if (op == OPCode.EXACTN_IC || op == OPCode.EXACTN_IC_SB) {
                 addLength(mbLength * strLength);
             } else {
-                addLength(strLength);                
+                addLength(strLength);
             }
         }
-        addBytes(bytes, p, mbLength * strLength);
+
+        if (Config.USE_STRING_TEMPLATES && opTemplated(op)) {
+            addInt(templateNum);
+            addInt(p);
+            addTemplate(bytes);
+        } else {
+            addBytes(bytes, p, mbLength * strLength);
+        }
     }
 
     private int compileLengthStringNode(Node node) {
         StringNode sn = (StringNode)node;
         if (sn.length() <= 0) return 0;
         boolean ambig = sn.isAmbig();
-        
+
         int p, prev;
         p = prev = sn.p;
         int end = sn.end;
         byte[]bytes = sn.bytes;
         int prevLen = enc.length(bytes, p, end);
         p += prevLen;
-        
+
         int slen = 1;
         int rlen = 0;
-        
+
         while (p < end) {
             int len = enc.length(bytes, p, end);
             if (len == prevLen) {
@@ -231,9 +258,9 @@ final class ArrayCompiler extends Compiler {
         rlen += r;
         return rlen;
     }
-    
+
     private int compileLengthStringRawNode(StringNode sn) {
-        if (sn.length() <= 0) return 0;        
+        if (sn.length() <= 0) return 0;
         return addCompileStringlength(sn.bytes, sn.p, 1 /*sb*/, sn.length(), false);
     }
 
@@ -241,7 +268,7 @@ final class ArrayCompiler extends Compiler {
         addLength(mbuf.used);
         addInts(mbuf.p, mbuf.used);
     }
-    
+
     private int compileLengthCClassNode(CClassNode cc) {
         if (cc.isShare()) return OPSize.OPCODE + OPSize.POINTER;
 
@@ -254,7 +281,7 @@ final class ArrayCompiler extends Compiler {
             } else {
                 len = OPSize.OPCODE + BitSet.BITSET_SIZE;
             }
-            
+
             len += OPSize.LENGTH + cc.mbuf.used;
         }
         return len;
@@ -267,7 +294,7 @@ final class ArrayCompiler extends Compiler {
             addPointer(cc);
             return;
         }
-        
+
         if (cc.mbuf == null) {
             if (cc.isNot()) {
                 addOpcode(enc.isSingleByte() ? OPCode.CCLASS_NOT_SB : OPCode.CCLASS_NOT);
@@ -282,7 +309,7 @@ final class ArrayCompiler extends Compiler {
                 } else {
                     addOpcode(OPCode.CCLASS_MB);
                 }
-                addMultiByteCClass(cc.mbuf);                
+                addMultiByteCClass(cc.mbuf);
             } else {
                 if (cc.isNot()) {
                     addOpcode(OPCode.CCLASS_MIX_NOT);
@@ -308,17 +335,17 @@ final class ArrayCompiler extends Compiler {
                 op = enc.isSingleByte() ? OPCode.WORD_SB : OPCode.WORD;
             }
             break;
-            
+
         default:
             newInternalException(ERR_PARSER_BUG);
-            return; // not reached  
+            return; // not reached
         } // inner switch
         addOpcode(op);
     }
 
     @Override
     protected void compileAnyCharNode() {
-        if (isMultiline(regex.options)) {            
+        if (isMultiline(regex.options)) {
             addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_ML_SB : OPCode.ANYCHAR_ML);
         } else {
             addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_SB : OPCode.ANYCHAR);
@@ -328,7 +355,7 @@ final class ArrayCompiler extends Compiler {
     @Override
     protected void compileCallNode(CallNode node) {
         addOpcode(OPCode.CALL);
-        node.unsetAddrList.add(regex.codeLength, node.target);
+        node.unsetAddrList.add(codeLength, node.target);
         addAbsAddr(0); /*dummy addr.*/
     }
 
@@ -341,7 +368,7 @@ final class ArrayCompiler extends Compiler {
             addLength(br.nestLevel);
             // !goto add_bacref_mems;!
             addLength(br.backNum);
-            for (int i=br.backNum-1; i>=0; i--) addMemNum(br.back[i]);                        
+            for (int i=br.backNum-1; i>=0; i--) addMemNum(br.back[i]);
             return;
         } else { // USE_BACKREF_AT_LEVEL
             if (br.backNum == 1) {
@@ -370,7 +397,7 @@ final class ArrayCompiler extends Compiler {
                 }
                 // !add_bacref_mems:!
                 addLength(br.backNum);
-                for (int i=br.backNum-1; i>=0; i--) addMemNum(br.back[i]);                        
+                for (int i=br.backNum-1; i>=0; i--) addMemNum(br.back[i]);
             }
         }
     }
@@ -388,45 +415,46 @@ final class ArrayCompiler extends Compiler {
             System.arraycopy(regex.repeatRangeHi, 0, tmp, 0, regex.repeatRangeHi.length);
             regex.repeatRangeHi = tmp;
         }
-        
+
         regex.repeatRangeLo[id] = lower;
         regex.repeatRangeHi[id] = isRepeatInfinite(upper) ? 0x7fffffff : upper;
     }
-    
+
     private void compileRangeRepeatNode(QuantifierNode qn, int targetLen, int emptyInfo) {
+        regex.requireStack = true;
         int numRepeat = regex.numRepeat;
         addOpcode(qn.greedy ? OPCode.REPEAT : OPCode.REPEAT_NG);
         addMemNum(numRepeat); /* OP_REPEAT ID */
         regex.numRepeat++;
         addRelAddr(targetLen + OPSize.REPEAT_INC);
-        
+
         entryRepeatRange(numRepeat, qn.lower, qn.upper);
-        
+
         compileTreeEmptyCheck(qn.target, emptyInfo);
-        
+
         if ((Config.USE_SUBEXP_CALL && regex.numCall > 0) || qn.isInRepeat()) {
             addOpcode(qn.greedy ? OPCode.REPEAT_INC_SG : OPCode.REPEAT_INC_NG_SG);
         } else {
-            addOpcode(qn.greedy ? OPCode.REPEAT_INC : OPCode.REPEAT_INC_NG);            
+            addOpcode(qn.greedy ? OPCode.REPEAT_INC : OPCode.REPEAT_INC_NG);
         }
-        
+
         addMemNum(numRepeat); /* OP_REPEAT ID */
     }
-    
+
     private static final int QUANTIFIER_EXPAND_LIMIT_SIZE   = 50; // was 50
 
-    private static boolean cknOn(int ckn) { 
+    private static boolean cknOn(int ckn) {
         return ckn > 0;
     }
-    
+
     private int compileCECLengthQuantifierNode(QuantifierNode qn) {
         boolean infinite = isRepeatInfinite(qn.upper);
         int emptyInfo = qn.targetEmptyInfo;
-        
+
         int tlen = compileLengthTree(qn.target);
         int ckn = regex.numCombExpCheck > 0 ? qn.combExpCheckNum : 0;
         int cklen = cknOn(ckn) ? OPSize.STATE_CHECK_NUM : 0;
-        
+
         /* anychar repeat */
         if (qn.target.getType() == NodeType.CANY) {
             if (qn.greedy && infinite) {
@@ -437,14 +465,14 @@ final class ArrayCompiler extends Compiler {
                 }
             }
         }
-        
+
         int modTLen;
         if (emptyInfo != 0) {
             modTLen = tlen + (OPSize.NULL_CHECK_START + OPSize.NULL_CHECK_END);
         } else {
             modTLen = tlen;
         }
-        
+
         int len;
         if (infinite && qn.lower <= 1) {
             if (qn.greedy) {
@@ -482,7 +510,7 @@ final class ArrayCompiler extends Compiler {
             len = OPSize.PUSH + cklen + OPSize.JUMP + tlen;
         } else {
             len = OPSize.REPEAT_INC + modTLen + OPSize.OPCODE + OPSize.RELADDR + OPSize.MEMNUM;
-            
+
             if (cknOn(ckn)) {
                 len += OPSize.STATE_CHECK;
             }
@@ -492,13 +520,14 @@ final class ArrayCompiler extends Compiler {
 
     @Override
     protected void compileCECQuantifierNode(QuantifierNode qn) {
+        regex.requireStack = true;
         boolean infinite = isRepeatInfinite(qn.upper);
         int emptyInfo = qn.targetEmptyInfo;
-        
+
         int tlen = compileLengthTree(qn.target);
-        
+
         int ckn = regex.numCombExpCheck > 0 ? qn.combExpCheckNum : 0;
-        
+
         if (qn.isAnyCharStar()) {
             compileTreeNTimes(qn.target, qn.lower);
             if (qn.nextHeadExact != null && !cknOn(ckn)) {
@@ -533,7 +562,7 @@ final class ArrayCompiler extends Compiler {
                 return;
             }
         }
-        
+
         int modTLen;
         if (emptyInfo != 0) {
             modTLen = tlen + (OPSize.NULL_CHECK_START + OPSize.NULL_CHECK_END);
@@ -594,7 +623,7 @@ final class ArrayCompiler extends Compiler {
             } else {
                 addOpcodeRelAddr(OPCode.PUSH, OPSize.JUMP);
             }
-            
+
             addOpcodeRelAddr(OPCode.JUMP, tlen);
             compileTree(qn.target);
         } else {
@@ -609,7 +638,7 @@ final class ArrayCompiler extends Compiler {
     private int compileNonCECLengthQuantifierNode(QuantifierNode qn) {
         boolean infinite = isRepeatInfinite(qn.upper);
         int emptyInfo = qn.targetEmptyInfo;
-        
+
         int tlen = compileLengthTree(qn.target);
 
         /* anychar repeat */
@@ -622,14 +651,14 @@ final class ArrayCompiler extends Compiler {
                 }
             }
         }
-        
+
         int modTLen = 0;
         if (emptyInfo != 0) {
             modTLen = tlen + (OPSize.NULL_CHECK_START + OPSize.NULL_CHECK_END);
         } else {
             modTLen = tlen;
         }
-        
+
         int len;
         if (infinite && (qn.lower <= 1 || tlen * qn.lower <= QUANTIFIER_EXPAND_LIMIT_SIZE)) {
             if (qn.lower == 1 && tlen > QUANTIFIER_EXPAND_LIMIT_SIZE) {
@@ -637,7 +666,7 @@ final class ArrayCompiler extends Compiler {
             } else {
                 len = tlen * qn.lower;
             }
-            
+
             if (qn.greedy) {
                 if (qn.headExact != null) {
                     len += OPSize.PUSH_OR_JUMP_EXACT1 + modTLen + OPSize.JUMP;
@@ -647,13 +676,13 @@ final class ArrayCompiler extends Compiler {
                     len += OPSize.PUSH + modTLen + OPSize.JUMP;
                 }
             } else {
-                len += OPSize.JUMP + modTLen + OPSize.PUSH; 
+                len += OPSize.JUMP + modTLen + OPSize.PUSH;
             }
-            
+
         } else if (qn.upper == 0 && qn.isRefered) { /* /(?<n>..){0}/ */
             len = OPSize.JUMP + tlen;
         } else if (!infinite && qn.greedy &&
-                  (qn.upper == 1 || (tlen + OPSize.PUSH) * qn.upper <= QUANTIFIER_EXPAND_LIMIT_SIZE )) { 
+                  (qn.upper == 1 || (tlen + OPSize.PUSH) * qn.upper <= QUANTIFIER_EXPAND_LIMIT_SIZE )) {
             len = tlen * qn.lower;
             len += (OPSize.PUSH + tlen) * (qn.upper - qn.lower);
         } else if (!qn.greedy && qn.upper == 1 && qn.lower == 0) { /* '??' */
@@ -663,14 +692,15 @@ final class ArrayCompiler extends Compiler {
         }
         return len;
     }
-    
+
     @Override
     protected void compileNonCECQuantifierNode(QuantifierNode qn) {
+        regex.requireStack = true;
         boolean infinite = isRepeatInfinite(qn.upper);
         int emptyInfo = qn.targetEmptyInfo;
-        
-        int tlen = compileLengthTree(qn.target);        
-        
+
+        int tlen = compileLengthTree(qn.target);
+
         if (qn.isAnyCharStar()) {
             compileTreeNTimes(qn.target, qn.lower);
             if (qn.nextHeadExact != null) {
@@ -691,7 +721,7 @@ final class ArrayCompiler extends Compiler {
                 return;
             }
         }
-        
+
         int modTLen;
         if (emptyInfo != 0) {
             modTLen = tlen + (OPSize.NULL_CHECK_START + OPSize.NULL_CHECK_END);
@@ -714,7 +744,7 @@ final class ArrayCompiler extends Compiler {
             } else {
                 compileTreeNTimes(qn.target, qn.lower);
             }
-                
+
             if (qn.greedy) {
                 if (qn.headExact != null) {
                     addOpcodeRelAddr(OPCode.PUSH_OR_JUMP_EXACT1, modTLen + OPSize.JUMP);
@@ -722,7 +752,7 @@ final class ArrayCompiler extends Compiler {
                     addBytes(sn.bytes, sn.p, 1);
                     compileTreeEmptyCheck(qn.target, emptyInfo);
                     addOpcodeRelAddr(OPCode.JUMP, -(modTLen + OPSize.JUMP + OPSize.PUSH_OR_JUMP_EXACT1));
-                } else if (qn.nextHeadExact != null) { 
+                } else if (qn.nextHeadExact != null) {
                     addOpcodeRelAddr(OPCode.PUSH_IF_PEEK_NEXT, modTLen + OPSize.JUMP);
                     StringNode sn = (StringNode)qn.nextHeadExact;
                     addBytes(sn.bytes, sn.p, 1);
@@ -741,11 +771,11 @@ final class ArrayCompiler extends Compiler {
         } else if (qn.upper == 0 && qn.isRefered) { /* /(?<n>..){0}/ */
             addOpcodeRelAddr(OPCode.JUMP, tlen);
             compileTree(qn.target);
-        } else if (!infinite && qn.greedy && 
+        } else if (!infinite && qn.greedy &&
                   (qn.upper == 1 || (tlen + OPSize.PUSH) * qn.upper <= QUANTIFIER_EXPAND_LIMIT_SIZE)) {
             int n = qn.upper - qn.lower;
             compileTreeNTimes(qn.target, qn.lower);
-            
+
             for (int i=0; i<n; i++) {
                 addOpcodeRelAddr(OPCode.PUSH, (n - i) * tlen + (n - i - 1) * OPSize.PUSH);
                 compileTree(qn.target);
@@ -764,7 +794,7 @@ final class ArrayCompiler extends Compiler {
         regex.options = node.option;
         int tlen = compileLengthTree(node.target);
         regex.options = prev;
-        
+
         if (isDynamic(prev ^ node.option)) {
             return OPSize.SET_OPTION_PUSH + OPSize.SET_OPTION + OPSize.FAIL + tlen + OPSize.SET_OPTION;
         } else {
@@ -775,17 +805,17 @@ final class ArrayCompiler extends Compiler {
     @Override
     protected void compileOptionNode(EncloseNode node) {
         int prev = regex.options;
-        
+
         if (isDynamic(prev ^ node.option)) {
             addOpcodeOption(OPCode.SET_OPTION_PUSH, node.option);
             addOpcodeOption(OPCode.SET_OPTION, prev);
             addOpcode(OPCode.FAIL);
         }
-        
+
         regex.options = node.option;
         compileTree(node.target);
         regex.options = prev;
-        
+
         if (isDynamic(prev ^ node.option)) {
             addOpcodeOption(OPCode.SET_OPTION, prev);
         }
@@ -795,14 +825,14 @@ final class ArrayCompiler extends Compiler {
         if (node.isOption()) {
             return compileLengthOptionNode(node);
         }
-        
+
         int tlen;
         if (node.target != null) {
             tlen = compileLengthTree(node.target);
         } else {
             tlen = 0;
         }
-        
+
         int len;
         switch (node.type) {
         case EncloseType.MEMORY:
@@ -822,17 +852,32 @@ final class ArrayCompiler extends Compiler {
                 len += tlen + (bsAt(regex.btMemEnd, node.regNum) ? OPSize.MEMORY_END_PUSH : OPSize.MEMORY_END);
             }
             break;
-            
+
         case EncloseType.STOP_BACKTRACK:
             if (node.isStopBtSimpleRepeat()) {
                 QuantifierNode qn = (QuantifierNode)node.target;
                 tlen = compileLengthTree(qn.target);
                 len = tlen * qn.lower + OPSize.PUSH + tlen + OPSize.POP + OPSize.JUMP;
-            } else { 
+            } else {
                 len = OPSize.PUSH_STOP_BT + tlen + OPSize.POP_STOP_BT;
             }
             break;
-            
+
+        case EncloseType.CONDITION:
+            len = OPSize.CONDITION;
+            if (node.target.getType() == NodeType.ALT) {
+                ConsAltNode x = (ConsAltNode)node.target;
+                tlen = compileLengthTree(x.car); /* yes-node */
+                len += tlen + OPSize.JUMP;
+                if (x.cdr == null) newInternalException(ERR_PARSER_BUG);
+                x = x.cdr;
+                tlen = compileLengthTree(x.car); /* no-node */
+                len += tlen;
+                if (x.cdr != null) newSyntaxException(ERR_INVALID_CONDITION_PATTERN);
+            } else {
+                newInternalException(ERR_PARSER_BUG);
+            }
+            break;
         default:
             newInternalException(ERR_PARSER_BUG);
             return 0; // not reached
@@ -847,8 +892,9 @@ final class ArrayCompiler extends Compiler {
         case EncloseType.MEMORY:
             if (Config.USE_SUBEXP_CALL) {
                 if (node.isCalled()) {
+                    regex.requireStack = true;
                     addOpcode(OPCode.CALL);
-                    node.callAddr = regex.codeLength + OPSize.ABSADDR + OPSize.JUMP;
+                    node.callAddr = codeLength + OPSize.ABSADDR + OPSize.JUMP;
                     node.setAddrFixed();
                     addAbsAddr(node.callAddr);
                     len = compileLengthTree(node.target);
@@ -861,16 +907,17 @@ final class ArrayCompiler extends Compiler {
                     addOpcodeRelAddr(OPCode.JUMP, len);
                 }
             } // USE_SUBEXP_CALL
-            
+
             if (bsAt(regex.btMemStart, node.regNum)) {
+                regex.requireStack = true;
                 addOpcode(OPCode.MEMORY_START_PUSH);
             } else {
                 addOpcode(OPCode.MEMORY_START);
             }
-            
+
             addMemNum(node.regNum);
             compileTree(node.target);
-            
+
             if (Config.USE_SUBEXP_CALL && node.isCalled()) {
                 if (bsAt(regex.btMemEnd, node.regNum)) {
                     addOpcode(node.isRecursion() ? OPCode.MEMORY_END_PUSH_REC : OPCode.MEMORY_END_PUSH);
@@ -888,14 +935,15 @@ final class ArrayCompiler extends Compiler {
                 addMemNum(node.regNum);
             }
             break;
-            
+
         case EncloseType.STOP_BACKTRACK:
+            regex.requireStack = true;
             if (node.isStopBtSimpleRepeat()) {
                 QuantifierNode qn = (QuantifierNode)node.target;
-                
+
                 compileTreeNTimes(qn.target, qn.lower);
-                
-                len = compileLengthTree(qn.target);                
+
+                len = compileLengthTree(qn.target);
                 addOpcodeRelAddr(OPCode.PUSH, len + OPSize.POP + OPSize.JUMP);
                 compileTree(qn.target);
                 addOpcode(OPCode.POP);
@@ -906,13 +954,34 @@ final class ArrayCompiler extends Compiler {
                 addOpcode(OPCode.POP_STOP_BT);
             }
             break;
-            
+
+        case EncloseType.CONDITION:
+            addOpcode(OPCode.CONDITION);
+            addMemNum(node.regNum);
+            if (node.target.getType() == NodeType.ALT) {
+                ConsAltNode x = (ConsAltNode)node.target;
+                len = compileLengthTree(x.car); /* yes-node */
+                if (x.cdr == null) newInternalException(ERR_PARSER_BUG);
+                x = x.cdr;
+                int len2 = compileLengthTree(x.car); /* no-node */
+                if (x.cdr != null) newSyntaxException(ERR_INVALID_CONDITION_PATTERN);
+                x = (ConsAltNode)node.target;
+                addRelAddr(len + OPSize.JUMP);
+                compileTree(x.car); /* yes-node */
+                addOpcodeRelAddr(OPCode.JUMP, len2);
+                x = x.cdr;
+                compileTree(x.car); /* no-node */
+            } else {
+                newInternalException(ERR_PARSER_BUG);
+            }
+            break;
+
         default:
             newInternalException(ERR_PARSER_BUG);
             break;
         } // switch
     }
-    
+
     private int compileLengthAnchorNode(AnchorNode node) {
         int tlen;
         if (node.target != null) {
@@ -920,25 +989,25 @@ final class ArrayCompiler extends Compiler {
         } else {
             tlen = 0;
         }
-        
+
         int len;
         switch (node.type) {
         case AnchorType.PREC_READ:
             len = OPSize.PUSH_POS + tlen + OPSize.POP_POS;
             break;
-        
+
         case AnchorType.PREC_READ_NOT:
             len = OPSize.PUSH_POS_NOT + tlen + OPSize.FAIL_POS;
             break;
-        
+
         case AnchorType.LOOK_BEHIND:
             len = OPSize.LOOK_BEHIND + tlen;
             break;
-        
+
         case AnchorType.LOOK_BEHIND_NOT:
             len = OPSize.PUSH_LOOK_BEHIND_NOT + tlen + OPSize.FAIL_LOOK_BEHIND_NOT;
             break;
-            
+
         default:
             len = OPSize.OPCODE;
             break;
@@ -950,46 +1019,52 @@ final class ArrayCompiler extends Compiler {
     protected void compileAnchorNode(AnchorNode node) {
         int len;
         int n;
-        
+
         switch (node.type) {
         case AnchorType.BEGIN_BUF:          addOpcode(OPCode.BEGIN_BUF);            break;
         case AnchorType.END_BUF:            addOpcode(OPCode.END_BUF);              break;
-        case AnchorType.BEGIN_LINE:         addOpcode(OPCode.BEGIN_LINE);           break;        
+        case AnchorType.BEGIN_LINE:         addOpcode(OPCode.BEGIN_LINE);           break;
         case AnchorType.END_LINE:           addOpcode(OPCode.END_LINE);             break;
         case AnchorType.SEMI_END_BUF:       addOpcode(OPCode.SEMI_END_BUF);         break;
-        case AnchorType.BEGIN_POSITION:     addOpcode(OPCode.BEGIN_POSITION);       break;        
+        case AnchorType.BEGIN_POSITION:     addOpcode(OPCode.BEGIN_POSITION);       break;
 
-        case AnchorType.WORD_BOUND:         
+        case AnchorType.WORD_BOUND:
             addOpcode(enc.isSingleByte() ? OPCode.WORD_BOUND_SB : OPCode.WORD_BOUND);
             break;
-            
+
         case AnchorType.NOT_WORD_BOUND:
             addOpcode(enc.isSingleByte() ? OPCode.NOT_WORD_BOUND_SB : OPCode.NOT_WORD_BOUND);
             break;
-        
+
         case AnchorType.WORD_BEGIN:
             if (Config.USE_WORD_BEGIN_END)
                 addOpcode(enc.isSingleByte() ? OPCode.WORD_BEGIN_SB : OPCode.WORD_BEGIN);
             break;
-        
+
         case AnchorType.WORD_END:
             if (Config.USE_WORD_BEGIN_END)
                 addOpcode(enc.isSingleByte() ? OPCode.WORD_END_SB : OPCode.WORD_END);
-            break;        
-            
+            break;
+
+        case AnchorType.KEEP:
+            addOpcode(OPCode.KEEP);
+            break;
+
         case AnchorType.PREC_READ:
+            regex.requireStack = true;
             addOpcode(OPCode.PUSH_POS);
             compileTree(node.target);
             addOpcode(OPCode.POP_POS);
             break;
 
         case AnchorType.PREC_READ_NOT:
+            regex.requireStack = true;
             len = compileLengthTree(node.target);
             addOpcodeRelAddr(OPCode.PUSH_POS_NOT, len + OPSize.FAIL_POS);
             compileTree(node.target);
             addOpcode(OPCode.FAIL_POS);
             break;
-            
+
         case AnchorType.LOOK_BEHIND:
             addOpcode(enc.isSingleByte() ? OPCode.LOOK_BEHIND_SB : OPCode.LOOK_BEHIND);
             if (node.charLength < 0) {
@@ -1001,8 +1076,9 @@ final class ArrayCompiler extends Compiler {
             addLength(n);
             compileTree(node.target);
             break;
-            
+
         case AnchorType.LOOK_BEHIND_NOT:
+            regex.requireStack = true;
             len = compileLengthTree(node.target);
             addOpcodeRelAddr(OPCode.PUSH_LOOK_BEHIND_NOT, len + OPSize.FAIL_LOOK_BEHIND_NOT);
             if (node.charLength < 0) {
@@ -1015,7 +1091,7 @@ final class ArrayCompiler extends Compiler {
             compileTree(node.target);
             addOpcode(OPCode.FAIL_LOOK_BEHIND_NOT);
             break;
-            
+
         default:
             newInternalException(ERR_PARSER_BUG);
         } // switch
@@ -1023,7 +1099,7 @@ final class ArrayCompiler extends Compiler {
 
     private int compileLengthTree(Node node) {
         int len = 0;
-        
+
         switch (node.getType()) {
         case NodeType.LIST:
             ConsAltNode lin = (ConsAltNode)node;
@@ -1031,7 +1107,7 @@ final class ArrayCompiler extends Compiler {
                 len += compileLengthTree(lin.car);
             } while ((lin = lin.cdr) != null);
             break;
-            
+
         case NodeType.ALT:
             ConsAltNode aln = (ConsAltNode)node;
             int n = 0;
@@ -1041,7 +1117,7 @@ final class ArrayCompiler extends Compiler {
             } while ((aln = aln.cdr) != null);
             len += (OPSize.PUSH + OPSize.JUMP) * (n - 1);
             break;
-            
+
         case NodeType.STR:
             StringNode sn = (StringNode)node;
             if (sn.isRaw()) {
@@ -1050,21 +1126,21 @@ final class ArrayCompiler extends Compiler {
                 len = compileLengthStringNode(sn);
             }
             break;
-            
+
         case NodeType.CCLASS:
             len = compileLengthCClassNode((CClassNode)node);
             break;
-            
+
         case NodeType.CTYPE:
         case NodeType.CANY:
             len = OPSize.OPCODE;
             break;
-            
+
         case NodeType.BREF:
             BackRefNode br = (BackRefNode)node;
-            
+
             if (Config.USE_BACKREF_WITH_LEVEL && br.isNestLevel()) {
-                len = OPSize.OPCODE + OPSize.OPTION + OPSize.LENGTH + 
+                len = OPSize.OPCODE + OPSize.OPTION + OPSize.LENGTH +
                       OPSize.LENGTH + (OPSize.MEMNUM * br.backNum);
             } else { // USE_BACKREF_AT_LEVEL
                 if (br.backNum == 1) {
@@ -1075,14 +1151,14 @@ final class ArrayCompiler extends Compiler {
                 }
             }
             break;
-            
+
         case NodeType.CALL:
             if (Config.USE_SUBEXP_CALL) {
                 len = OPSize.CALL;
                 break;
             } // USE_SUBEXP_CALL
             break;
-            
+
         case NodeType.QTFR:
             if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
                 len = compileCECLengthQuantifierNode((QuantifierNode)node);
@@ -1090,111 +1166,73 @@ final class ArrayCompiler extends Compiler {
                 len = compileNonCECLengthQuantifierNode((QuantifierNode)node);
             }
             break;
-            
+
         case NodeType.ENCLOSE:
             len = compileLengthEncloseNode((EncloseNode)node);
             break;
-            
+
         case NodeType.ANCHOR:
             len = compileLengthAnchorNode((AnchorNode)node);
             break;
 
         default:
             newInternalException(ERR_PARSER_BUG);
-            
+
         } //switch
         return len;
     }
 
     private void ensure(int size) {
-        if (size >= regex.code.length) {
-            int length = regex.code.length << 1;
+        if (size >= code.length) {
+            int length = code.length << 1;
             while (length <= size) length <<= 1;
             int[]tmp = new int[length];
-            System.arraycopy(regex.code, 0, tmp, 0, regex.code.length);
-            regex.code = tmp;
+            System.arraycopy(code, 0, tmp, 0, code.length);
+            code = tmp;
         }
     }
-    
+
     private void addInt(int i) {
-        if (regex.codeLength >= regex.code.length) {
-            int[]tmp = new int[regex.code.length << 1];
-            System.arraycopy(regex.code, 0, tmp, 0, regex.code.length);
-            regex.code = tmp;
+        if (codeLength >= code.length) {
+            int[]tmp = new int[code.length << 1];
+            System.arraycopy(code, 0, tmp, 0, code.length);
+            code = tmp;
         }
-        regex.code[regex.codeLength++] = i;
+        code[codeLength++] = i;
     }
-    
+
     void setInt(int i, int offset) {
         ensure(offset);
         regex.code[offset] = i;
     }
-    
+
     private void addObject(Object o) {
         if (regex.operands == null) {
             regex.operands = new Object[4];
         } else if (regex.operandLength >= regex.operands.length) {
-            Object[]tmp = new Object[regex.operands.length << 1];            
-            System.arraycopy(regex.operands, 0, tmp, 0, regex.operands.length);            
-            regex.operands = tmp;            
+            Object[]tmp = new Object[regex.operands.length << 1];
+            System.arraycopy(regex.operands, 0, tmp, 0, regex.operands.length);
+            regex.operands = tmp;
         }
         addInt(regex.operandLength);
         regex.operands[regex.operandLength++] = o;
     }
-    
+
     private void addBytes(byte[]bytes, int p ,int length) {
-        ensure(regex.codeLength + length);
+        ensure(codeLength + length);
         int end = p + length;
-        
-        while (p < end) regex.code[regex.codeLength++] = bytes[p++];
+
+        while (p < end) code[codeLength++] = bytes[p++];
     }
-    
-    private void addInts(int[]ints, int length) { 
-        ensure(regex.codeLength + length);
-        System.arraycopy(ints, 0, regex.code, regex.codeLength, length);
-        regex.codeLength += length;
+
+    private void addInts(int[]ints, int length) {
+        ensure(codeLength + length);
+        System.arraycopy(ints, 0, code, codeLength, length);
+        codeLength += length;
     }
-    
+
     private void addOpcode(int opcode) {
         addInt(opcode);
-        
-        switch(opcode) {
-        case OPCode.ANYCHAR_STAR:
-        case OPCode.ANYCHAR_STAR_SB:
-        case OPCode.ANYCHAR_ML_STAR:
-        case OPCode.ANYCHAR_ML_STAR_SB:
-        case OPCode.ANYCHAR_STAR_PEEK_NEXT:
-        case OPCode.ANYCHAR_STAR_PEEK_NEXT_SB:
-        case OPCode.ANYCHAR_ML_STAR_PEEK_NEXT:            
-        case OPCode.ANYCHAR_ML_STAR_PEEK_NEXT_SB:
-        case OPCode.STATE_CHECK_ANYCHAR_STAR:
-        case OPCode.STATE_CHECK_ANYCHAR_STAR_SB:
-        case OPCode.STATE_CHECK_ANYCHAR_ML_STAR:            
-        case OPCode.MEMORY_START_PUSH:
-        case OPCode.MEMORY_END_PUSH:
-        case OPCode.MEMORY_END_PUSH_REC:
-        case OPCode.MEMORY_END_REC:
-        case OPCode.NULL_CHECK_START:
-        case OPCode.NULL_CHECK_END_MEMST_PUSH:
-        case OPCode.PUSH:
-        case OPCode.STATE_CHECK_PUSH:
-        case OPCode.STATE_CHECK_PUSH_OR_JUMP:
-        case OPCode.STATE_CHECK:
-        case OPCode.PUSH_OR_JUMP_EXACT1:
-        case OPCode.PUSH_IF_PEEK_NEXT:
-        case OPCode.REPEAT:
-        case OPCode.REPEAT_NG:
-        case OPCode.REPEAT_INC_SG:
-        case OPCode.REPEAT_INC_NG:
-        case OPCode.REPEAT_INC_NG_SG:
-        case OPCode.PUSH_POS:            
-        case OPCode.PUSH_POS_NOT:
-        case OPCode.PUSH_STOP_BT:
-        case OPCode.PUSH_LOOK_BEHIND_NOT:
-        case OPCode.CALL:
-        case OPCode.RETURN: // it will appear only with CALL though
-            regex.stackNeeded = true;
-        }
     }
 
     private void addStateCheckNum(int num) {
@@ -1233,5 +1271,16 @@ final class ArrayCompiler extends Compiler {
     private void addOpcodeOption(int opcode, int option) {
         addOpcode(opcode);
         addOption(option);
-    }    
+    }
+
+    private void addTemplate(byte[]bytes) {
+        if (templateNum == 0) {
+            templates = new byte[2][];
+        } else if (templateNum == templates.length) {
+            byte[][]tmp = new byte[templateNum * 2][];
+            System.arraycopy(templates, 0, tmp, 0, templateNum);
+            templates = tmp;
+        }
+        templates[templateNum++] = bytes;
+    }
 }
