@@ -1,20 +1,20 @@
 /*
- * Permission is hereby granted, free of charge, to any person obtaining a copy of 
- * this software and associated documentation files (the "Software"), to deal in 
- * the Software without restriction, including without limitation the rights to 
- * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy of
+ * this software and associated documentation files (the "Software"), to deal in
+ * the Software without restriction, including without limitation the rights to
+ * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
  * of the Software, and to permit persons to whom the Software is furnished to do
  * so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included in all
  * copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
 package org.joni.test;
@@ -33,16 +33,16 @@ public class TestA extends Test {
     public Encoding encoding() {
         return ASCIIEncoding.INSTANCE;
     }
-    
+
     public String testEncoding() {
         return "iso-8859-2";
     }
 
     public Syntax syntax() {
         return Syntax.DEFAULT;
-    }   
+    }
 
-    public void test() {
+    public void test() throws InterruptedException {
         x2s("", "", 0, 0);
         x2s("^", "", 0, 0);
         x2s("$", "", 0, 0);
@@ -451,11 +451,11 @@ public class TestA extends Test {
         x2s("x((.)*)*x(?i:\\1)\\Z", "0x1x2x1X2", 1, 9);
         x2s("(?:()|()|()|()|()|())*\\2\\5", "", 0, 0);
         x2s("(?:()|()|()|(x)|()|())*\\2b\\5", "b", 0, 1);
-        
+
         x3s("\\A(?<a>|.|(?:(?<b>.)\\g<a>\\k<b+0>))\\z", "reer", 0, 4, 1);
         x3s("(?-i:\\g<name>)(?i:(?<name>a)){0}", "A", 0, 1, 1);
-        
-        String pat = 
+
+        String pat =
             "(?<element> \\g<stag> \\g<content>* \\g<etag> ){0}" +
             "(?<stag> < \\g<name> \\s* > ){0}" +
             "(?<name> [a-zA-Z_:]+ ){0}" +
@@ -464,7 +464,7 @@ public class TestA extends Test {
             "\\g<element>";
 
         String str = "<foo>f<bar>bbb</bar>f</foo>";
-        
+
         x3s(pat, str, 0, 27, 0, Option.EXTEND);
         x3s(pat, str, 0, 27, 1, Option.EXTEND);
         x3s(pat, str, 6, 11, 2, Option.EXTEND);
@@ -473,8 +473,63 @@ public class TestA extends Test {
         x3s(pat, str, 21, 27, 5, Option.EXTEND);
 
         x2s("(a)b\\k<1>", "aba", 0, 3);
+        x2s("^(?>(?=a)(a|))++$", "a", 0, 1);
+        x2s("\\k", "k", 0, 1);
+        x2s("\\kx", "kx", 0, 2);
+        x2s("\\g", "g", 0, 1);
+        x2s("\\gx", "gx", 0, 2);
+        x2s("\\k\\g", "kg", 0, 2);
+        ns("\\00", "00");
+        ns("\\70", "70");
+        x2s("\\80", "80", 0, 2);
+        x2s("\\90", "90", 0, 2);
+
+        ns("(?<!b|aa)c", "Aac", Option.IGNORECASE);
+        x2s("(?<!b|aa)", "Aac", 0, 0, Option.IGNORECASE);
+        x2s("(?<=b|aa)c", "Aac", 2, 3, Option.IGNORECASE);
+        x2s("(?<=b|aa)", "Aac", 2, 2, Option.IGNORECASE);
+
+        ns("\\A[a-f&&[^b-c]&&[^e]]\\z", "e");
+        ns("[[^a]&&e&&[^e]]", "e");
+
+        x2s("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", 0, 35, Option.IGNORECASE);
+        x2s("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA", 0, 35, Option.IGNORECASE);
+        x2s("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "aaaaaaaaaaaaaaaaaaAAAAAAAAAAAAAAAAA", 0, 35, Option.IGNORECASE);
+
+        x2s("(?mix)", "", 0, 0);
+
+        x2s("(?<=\\babc)d", " abcd", 4, 5);
+        x2s("(?<=\\Babc)d", "aabcd", 4, 5);
+        x2s("([^\\s]+),(.*)+", " xxxx,         xxx xxxxxx xxxxxxxxx xxxxxxx, xxxx xxxxx xxxxx ", 1, 62);
+        x2s(",(.*)+", " xxxx,         xxx xxxxxx xxxxxxxxx xxxxxxx, xxxx xxxxx xxxxx ", 5, 62);
+
+        x2s("(?:(a)|(b))(?(1)cd)e", "acde", 0, 4);
+        ns("(?:(a)|(b))(?(1)cd)e", "ae");
+        x2s("(?:(a)|(b))(?(2)cd)e", "ae", 0, 2);
+        ns("(?:(a)|(b))(?(2)cd)e", "acde");
+        x2s("(?:(a)|(b))(?(1)c|d)", "ac", 0, 2);
+        x2s("(?:(a)|(b))(?(1)c|d)", "bd", 0, 2);
+        ns("(?:(a)|(b))(?(1)c|d)", "ad");
+        ns("(?:(a)|(b))(?(1)c|d)", "bc");
+        x2s("(?:(a)|(b))(?:(?(1)cd)e|fg)", "acde", 0, 4);
+        x2s("(?:(a)|(b))(?:(?(1)cd|x)e|fg)", "bxe", 0, 3);
+        ns("(?:(a)|(b))(?:(?(2)cd|x)e|fg)", "bxe");
+        x2s("(?:(?<x>a)|(?<y>b))(?:(?(<x>)cd|x)e|fg)", "bxe", 0, 3);
+        ns("(?:(?<x>a)|(?<y>b))(?:(?(<y>)cd|x)e|fg)", "bxe");
+        x2s("((?<=a))?(?(1)b|c)", "abc", 1, 2);
+        x2s("((?<=a))?(?(1)b|c)", "bc", 1, 2);
+        x2s("((?<x>x)|(?<y>y))(?(<x>)y|x)", "xy", 0, 2);
+        x2s("((?<x>x)|(?<y>y))(?(<x>)y|x)", "yx", 0, 2);
+        ns("((?<x>x)|(?<y>y))(?(<x>)y|x)", "xx");
+        ns("((?<x>x)|(?<y>y))(?(<x>)y|x)", "yy");
+
+        x2s("\\R", "\n", 0, 1);
+        x2s("\\R", "\r", 0, 1);
+        x2s("\\R{3}", "\r\r\n\n", 0, 4);
+
+        x2s("\\X{5}", "ăăab\n", 0, 5);
     }
-    
+
     public static void main(String[] args) throws Throwable{
         new TestA().run();
     }
